﻿using System;
using System.Runtime.InteropServices;

namespace TVAppLauncher
{
    /// <summary>
    /// Base class toregister global windows hooks
    /// </summary>
    internal abstract class GlobalHookBase : IDisposable
    {
        protected enum HookType
        {
            // There are several other types but the low level ones 
            // are the only ones that can be used in .Net.
            // For the others a C or C++ dll is needed.
            Keyboard = 13, // WH_HEYBOARD_LL
            Mouse = 14     // WH_MOUSE_LL
        }

        private readonly IntPtr _Hook;

        private delegate int MessageDelegate(int code, IntPtr wparam, IntPtr lparam);

        // Keep a reference to the delegate so it isn't garbage collected.
        [MarshalAs(UnmanagedType.FunctionPtr)]
        private MessageDelegate _CallbackMethod;

        protected GlobalHookBase(HookType hookType, bool onThread = false)
        {
            _CallbackMethod = MessageDelegateMethod;

            var threadId = onThread ? GetCurrentThreadId() : IntPtr.Zero;
            _Hook = SetWindowsHookEx((int)hookType, _CallbackMethod, IntPtr.Zero, threadId);
        }

        protected abstract bool ProcessMessage(IntPtr wParam, IntPtr lParam);

        public void Dispose()
        {
            UnhookWindowsHookEx(_Hook);
        }

        private int MessageDelegateMethod(int hookcode, IntPtr wparam, IntPtr lparam)
        {
            const int HC_ACTION = 0;

            if (hookcode >= HC_ACTION && ProcessMessage(wparam, lparam))
                return 1;
            return CallNextHook(_Hook, hookcode, wparam, lparam);
        }


        [DllImport("user32", EntryPoint = "SetWindowsHookExA")]
        private static extern IntPtr SetWindowsHookEx(int idHook, Delegate lpfn, IntPtr hmod, IntPtr dwThreadId);

        [DllImport("user32", EntryPoint = "UnhookWindowsHookEx")]
        private static extern int UnhookWindowsHookEx(IntPtr hHook);

        [DllImport("user32", EntryPoint = "CallNextHookEx")]
        private static extern int CallNextHook(IntPtr hHook, int ncode, IntPtr wParam, IntPtr lParam);

        [DllImport("kernel32.dll")]
        private static extern IntPtr GetCurrentThreadId();
    }
}