﻿using System;
using System.Runtime.InteropServices;

namespace TVAppLauncher
{
    internal class GlobalKeyboardHook : GlobalHookBase
    {
        public GlobalKeyboardHook() : base(HookType.Keyboard)
        {
        }

        public event EventHandler<KeyEventArgs> KeyEvent;

        protected override bool ProcessMessage(IntPtr wParam, IntPtr lParam)
        {
            var data = (KeyboardHookStruct)Marshal.PtrToStructure(lParam, typeof(KeyboardHookStruct));

            var message = (WindowMessage)wParam;
            var altDown = (data.Flags & KeyboardHookStructFlags.AltDown) == KeyboardHookStructFlags.AltDown;
            var extended = (data.Flags & KeyboardHookStructFlags.Extended) == KeyboardHookStructFlags.Extended;
            //var injected = (data.Flags & KeyboardHookStructFlags.Injected) == KeyboardHookStructFlags.Injected;

            var args = new KeyEventArgs(message, (VirtualKeyCode)data.VirtualKeyCode, data.ScanCode, extended, altDown);
            KeyEvent?.Invoke(this, args);
            return args.Handled;
        }

        [Flags]
        private enum KeyboardHookStructFlags : uint
        {
            Extended = 0x01,
            Injected = 0x10,
            AltDown = 0x20,
            Up = 0x80,
        }

        // KBDLLHOOKSTRUCT 
        [StructLayout(LayoutKind.Sequential)]
        private class KeyboardHookStruct
        {
            public uint VirtualKeyCode;
            public uint ScanCode;
            public KeyboardHookStructFlags Flags;
            public uint Time;
            public UIntPtr ExtraInfo;
        }
    }

}
