﻿using System;

namespace TVAppLauncher
{
    internal interface IInputProcessor
    {
        void ReplaceKeyIfNeeded(ref VirtualKeyCode virtualKeyCode, ref uint scanCode, ref bool extended);
        bool ProcessGlobalKeyboardEvent(MainWindow mainWindow, KeyEventArgs eventArgs, IntPtr activeWindow, bool hasForeground);
        bool ProcessMCEButtonPress(MainWindow mainWindow, MCERemoteButtonCode buttonCode, IntPtr activeWindow, bool hasForeground);
    }

    internal class YouTubeTVInputProcessor : IInputProcessor
    {
        public void ReplaceKeyIfNeeded(ref VirtualKeyCode virtualKeyCode, ref uint scanCode, ref bool extended)
        {
            if (virtualKeyCode == VirtualKeyCode.BrowserBack)
            {
                virtualKeyCode = VirtualKeyCode.Escape;
                scanCode = 1;
                extended = false;
            }
        }

        public bool ProcessGlobalKeyboardEvent(MainWindow mainWindow, KeyEventArgs eventArgs, IntPtr activeWindow, bool hasForeground)
        {
            switch (eventArgs.VirtualKeyCode)
            {
                case VirtualKeyCode.Mute:
                    if(eventArgs.Message == WindowMessage.KeyDown)
                        NativeMethods.SendAppCommand(activeWindow, AppCommand.VolumeMute);
                    return eventArgs.Handled = true;
                case VirtualKeyCode.VolumeUp:
                    if (eventArgs.Message == WindowMessage.KeyDown)
                        NativeMethods.SendAppCommand(activeWindow, AppCommand.VolumeUp);
                    return eventArgs.Handled = true;                    
                case VirtualKeyCode.VolumeDown:
                    if (eventArgs.Message == WindowMessage.KeyDown)
                        NativeMethods.SendAppCommand(activeWindow, AppCommand.VolumeDown);
                    return eventArgs.Handled = true;
                case VirtualKeyCode.Next:
                case VirtualKeyCode.Previous:
                    if (eventArgs.Message == WindowMessage.KeyDown)
                        KeyboardHelper.SendKeyDown(activeWindow, eventArgs.VirtualKeyCode, eventArgs.ScanCode, eventArgs.Extended);
                    else if(eventArgs.Message == WindowMessage.KeyUp)
                        KeyboardHelper.SendKeyUp(activeWindow, eventArgs.VirtualKeyCode, eventArgs.ScanCode, eventArgs.Extended);
                    return eventArgs.Handled = true;
            }

            return false;
        }

        public bool ProcessMCEButtonPress(MainWindow mainWindow, MCERemoteButtonCode buttonCode, IntPtr activeWindow, bool hasForeground)
        {
            switch (buttonCode)
            {
                // These buttons don't send cooresponding key events.
                case MCERemoteButtonCode.Pause:
                    KeyboardHelper.SendKeyDownKeyUp(activeWindow, VirtualKeyCode.Space, 0x39, false);
                    return true;
                case MCERemoteButtonCode.Play:
                    KeyboardHelper.SendKeyDownKeyUp(activeWindow, VirtualKeyCode.Return, 0x1C, false);
                    return true;
                case MCERemoteButtonCode.Rewind:
                    KeyboardHelper.SendKeyDownKeyUp(activeWindow, VirtualKeyCode.J, 0x24, true);
                    return true;
                case MCERemoteButtonCode.FastForward:
                    KeyboardHelper.SendKeyDownKeyUp(activeWindow, VirtualKeyCode.L, 0x26, true);
                    return true;                
            }

            return false;
        }
    }
}
