﻿using System;
using System.Threading.Tasks;

namespace TVAppLauncher
{
    internal enum VirtualKeyCode
    {
        Tab = 0x09,
        Return = 0x0D,
        Escape = 0x1B,
        Space = 0x20,
        Right = 0x25,
        Up = 0x26,
        Left = 0x27,
        Down = 0x28,
        J = 0x4A,
        L = 0x4C,
        F4 = 0x73,
        F11 = 0x7A,
        PageUp = 0x9C,
        PageDown = 0x9D,
        Alt = 0xA4,
        BrowserBack = 0xA6,
        BrowserForward = 0xA7,
        Mute = 0xAD,
        VolumeDown = 0xAE,
        VolumeUp = 0xAF,
        Next = 0xB0,
        Previous = 0xB1,
        Stop = 0xB2,
        FastForward = 0xB3,
        Rewind = 0xB4,
    }

    internal static class KeyboardHelper
    {
        public static void SendKeyDown(IntPtr hWnd, VirtualKeyCode keyCode, uint scanCode, bool extended)
        {
            var lParam = (int)(0x00000001 | (scanCode << 16));
            if (extended)
                lParam |= 0x01000000;

            NativeMethods.PostMessage(hWnd, WindowMessage.KeyDown, (IntPtr)keyCode, (IntPtr)lParam);
        }

        public static void SendKeyUp(IntPtr hWnd, VirtualKeyCode keyCode, uint scanCode, bool extended)
        {
            var lParam = (int)(0xC0000001 | (scanCode << 16));
            if (extended)
                lParam |= 0x01000000;

            NativeMethods.PostMessage(hWnd, WindowMessage.KeyUp, (IntPtr)keyCode, (IntPtr)lParam);
        }

        public static async void SendKeyDownKeyUp(IntPtr hWnd, VirtualKeyCode keyCode, uint scanCode, bool extended)
        {
            await Task.Yield();

            var lParam = (int)(0x00000001 | (scanCode << 16));
            if (extended)
                lParam |= 0x01000000;

            NativeMethods.SendMessage(hWnd, WindowMessage.KeyDown, (IntPtr)keyCode, (IntPtr)lParam);

            await Task.Delay(50);

            lParam = (int)(lParam | 0xC0000000);
            NativeMethods.SendMessage(hWnd, WindowMessage.KeyUp, (IntPtr)keyCode, (IntPtr)lParam);
        }
    }
}
