﻿using System;
using System.Windows.Threading;

namespace TVAppLauncher
{
    internal static class LongPressHelper
    {
        private const int _HoldKeyDuration = 750;

        private static readonly DispatcherTimer _Timer = new DispatcherTimer(
            TimeSpan.FromMilliseconds(_HoldKeyDuration),
            DispatcherPriority.ContextIdle,
            Timer_Tick,
            Dispatcher.CurrentDispatcher);

        private static Action _LongPressAction;
        private static Action _ShortPressedAction;

        public static void RegisterPressed(Action longPressAction, Action shortPressedAction = null)
        {
            if (_LongPressAction != null || _ShortPressedAction != null)
                return;
            _LongPressAction = longPressAction;
            _ShortPressedAction = shortPressedAction;
            _Timer.Start();
        }

        /// <summary>
        /// Returns true if short pressed.
        /// </summary>
        public static bool RegisterReleased()
        {
            var shortPressed = _Timer != null && _Timer.IsEnabled;
            if (shortPressed)
            {
                _Timer.Stop();
                _ShortPressedAction?.Invoke();
            }
            _LongPressAction = _ShortPressedAction = null;
            return shortPressed;
        }

        private static void Timer_Tick(object sender, EventArgs e)
        {
            _Timer.Stop();
            _LongPressAction?.Invoke();
        }
    }

}
