﻿using System;
using System.Collections.Generic;
using System.IO;

namespace TVAppLauncher
{
    internal enum KnownExeIdentifier
    {
        Unknown,
        Chrome,
        Firefox,
        Vivaldi,
    }

    internal class StartupArguments
    {
        private const string _ChromeExePath = @"C:\Program Files (x86)\Google\Chrome\Application\chrome.exe";
        private const string _VivaldiExePath = "vivaldi.exe";
        private const string _YouTubeTVUrl = "https://www.YouTube.com/tv#";

        public StartupArguments(IEnumerable<string> args)
        {
            ParseArgs(args);
        }

        public KnownExeIdentifier ExeId { get; private set; }
        public string Exe { get; private set; }
        public string Url { get; private set; }
        public string Args { get; private set; }
        public bool Quit { get; private set; }
        public bool Start { get; private set; }
        public bool NoOpeningOverlay { get; private set; }
        public bool ForceFullScreen { get; private set; }
        public IInputProcessor InputProcessor { get; private set; }

        private void ParseArgs(IEnumerable<string> args)
        {
            foreach (var arg in args)
            {
                if (arg.Equals("youtube", StringComparison.CurrentCultureIgnoreCase))
                    Url = _YouTubeTVUrl;
                else if (arg.Equals("chrome", StringComparison.CurrentCultureIgnoreCase))
                    Exe = _ChromeExePath;
                else if (arg.Equals("vivaldi", StringComparison.CurrentCultureIgnoreCase))
                    Exe = _VivaldiExePath;
                else if (arg.StartsWith("exe:"))
                    Exe = arg.Substring(4);
                else if (arg.StartsWith("url:"))
                    Url = arg.Substring(4);
                else if (arg.StartsWith("www"))
                    Url = arg;
                else if (arg.StartsWith("http"))
                    Url = arg;
                else if (arg.EndsWith(".exe"))
                    Exe = arg;
                else if (arg.StartsWith("args:"))
                    Args = arg.Substring(5);
                else if (arg == "quit")
                    Quit = true;
                else if (arg == "start")
                    Start = true;
                else if (arg == "no-opening-overlay")
                    NoOpeningOverlay = true;
                else if (arg == "force-fullscreen")
                    ForceFullScreen = true;
                else if (Url == null && !arg.StartsWith("-"))
                    Url = arg;
            }

            if (Exe == null)
                Exe = File.Exists(_ChromeExePath) ? _ChromeExePath : "chrome.exe";

            if (Url == null)
                Url = _YouTubeTVUrl;

            ExeId = GetExeId(Exe);
            InputProcessor = GetInputProcessor(Url);
        }

        private static IInputProcessor GetInputProcessor(string url)
        {
            if (url == _YouTubeTVUrl)
                return new YouTubeTVInputProcessor();
            return null;
        }

        private static KnownExeIdentifier GetExeId(string exe)
        {
            if (exe.IndexOf("chrome", StringComparison.CurrentCultureIgnoreCase) >= 0)
                return KnownExeIdentifier.Chrome;
            if (exe.IndexOf("firefox", StringComparison.CurrentCultureIgnoreCase) >= 0)
                return KnownExeIdentifier.Firefox;
            if (exe.IndexOf("vivaldi", StringComparison.CurrentCultureIgnoreCase) >= 0)
                return KnownExeIdentifier.Vivaldi;
            return KnownExeIdentifier.Unknown;
        }
    }

}
